'use strict';

/* eslint-env node */

var errors = require('feathers-errors');
var debug = require('debug')('verify-reset:verifySignup');

var _require = require('./helpers'),
    getUserData = _require.getUserData,
    ensureObjPropsValid = _require.ensureObjPropsValid,
    ensureValuesAreStrings = _require.ensureValuesAreStrings,
    sanitizeUserForClient = _require.sanitizeUserForClient,
    notifier = _require.notifier;

module.exports.verifySignupWithLongToken = function (options, verifyToken) {
  return Promise.resolve().then(function () {
    ensureValuesAreStrings(verifyToken);

    return verifySignup(options, { verifyToken: verifyToken }, { verifyToken: verifyToken });
  });
};

module.exports.verifySignupWithShortToken = function (options, verifyShortToken, findUser) {
  return Promise.resolve().then(function () {
    ensureValuesAreStrings(verifyShortToken);
    ensureObjPropsValid(findUser, options.userPropsForShortToken);

    return verifySignup(options, findUser, { verifyShortToken: verifyShortToken });
  });
};

function verifySignup(options, query, tokens) {
  debug('verifySignup', query, tokens);
  var users = options.app.service(options.service);
  var usersIdName = users.id;

  return users.find({ query: query }).then(function (data) {
    return getUserData(data, ['isNotVerified', 'verifyNotExpired']);
  }).then(function (user) {
    if (!Object.keys(tokens).every(function (key) {
      return tokens[key] === user[key];
    })) {
      return patchUser(user, {
        verifyToken: null,
        verifyShortToken: null,
        verifyExpires: null
      }).then(function () {
        throw new errors.BadRequest('Invalid token. Get for a new one. (verify-reset)', { errors: { $className: 'badParam' } });
      });
    }

    return patchUser(user, {
      isVerified: user.verifyExpires > Date.now(),
      verifyToken: null,
      verifyShortToken: null,
      verifyExpires: null
    }).then(function (user1) {
      return notifier(options.notifier, 'verifySignup', user1);
    }).then(function (user1) {
      return sanitizeUserForClient(user1);
    });
  });

  function patchUser(user, patchToUser) {
    return users.patch(user[usersIdName], patchToUser, {}) // needs users from closure
    .then(function () {
      return Object.assign(user, patchToUser);
    });
  }
}